"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.create = exports.getParts = exports.isOptions = void 0;
const client_s3_1 = require("@aws-sdk/client-s3");
const mime_types_1 = require("mime-types");
const protopath_1 = require("protopath");
const Types = __importStar(require("./types"));
const utils_1 = require("./utils");
const apiVersion = '2014-11-06';
function isOptions(o) {
    if (typeof o === 'object' &&
        o !== null &&
        o !== undefined &&
        (0, utils_1.has)(o, 'access') &&
        (0, utils_1.has)(o, 'secret') &&
        typeof o.access === 'string' &&
        typeof o.secret === 'string') {
        return !(0, utils_1.has)(o, 'region') || typeof o.region === 'string';
    }
    return false;
}
exports.isOptions = isOptions;
function getS3(options) {
    if (!options)
        return new client_s3_1.S3Client({ apiVersion });
    const { region, access: accessKeyId, secret: secretAccessKey } = options;
    return new client_s3_1.S3Client({
        apiVersion,
        region,
        credentials: { accessKeyId, secretAccessKey },
    });
}
function getParts(path) {
    const parts = (0, protopath_1.popSlash)(path).split('/');
    const bucket = parts[0];
    if (!bucket)
        throw new Error('No S3 bucket supplied');
    const object = parts.slice(1).join('/');
    return [bucket, object.length ? object : undefined];
}
exports.getParts = getParts;
function create(options) {
    const s3 = getS3(options);
    async function read(path, { range } = {}) {
        const [Bucket, Key] = getParts(path);
        if (!Key)
            throw new Error('Invalid S3 read - no object specified');
        const command = new client_s3_1.GetObjectCommand({
            Bucket,
            Key,
            Range: range ? Types.Range.toHeaderValue(range) : undefined,
        });
        const { Body: body } = await s3.send(command);
        if (!body)
            throw new Error('Missing response body from S3');
        return drain(body);
    }
    async function write(path, data) {
        const [Bucket, Key] = getParts(path);
        if (!Key)
            throw new Error('Invalid S3 write - no object specified');
        const ContentType = (0, mime_types_1.lookup)(path) || undefined;
        const command = new client_s3_1.PutObjectCommand({
            Bucket,
            Key,
            Body: typeof data === 'string' ? data : Buffer.from(data),
            ContentType,
        });
        await s3.send(command);
    }
    async function drain(stream) {
        return await new Promise((resolve, reject) => {
            const chunks = [];
            stream.on('data', (chunk) => chunks.push(chunk));
            stream.on('error', reject);
            stream.on('end', () => resolve(Buffer.concat(chunks)));
        });
    }
    /*
    async function createReadStream(
      path: string,
      { range }: Types.ReadOptions = {}
    ) {
      const [Bucket, Key] = getParts(path)
      if (!Key) throw new Error('Invalid S3 read - no object specified')
  
      const options: S3.GetObjectRequest = { Bucket, Key }
      if (range) options.Range = Types.Range.toHeaderValue(range)
  
      const request = s3.getObject(options)
      return request.createReadStream().on('error', () => request.abort())
    }
    async function writeStream(path: string, data: NodeJS.ReadableStream) {
      const [Bucket, Key] = getParts(path)
      if (!Key) throw new Error('Invalid S3 write - no object specified')
      const ContentType = lookup(path) || undefined
      await s3.upload({ Bucket, Key, Body: data, ContentType }).promise()
    }
    */
    async function list(path) {
        const [Bucket, Key] = getParts(path);
        const Prefix = Key ? `${Key}/` : '';
        let ContinuationToken;
        let list = [];
        do {
            const command = new client_s3_1.ListObjectsV2Command({
                Bucket,
                Prefix,
                Delimiter: '/',
                ContinuationToken,
            });
            const response = await s3.send(command);
            if (!response.CommonPrefixes || !response.Contents) {
                throw new Error('Unexpected S3 list response');
            }
            list = [
                ...list,
                ...response.CommonPrefixes.map((v) => ({
                    type: 'directory',
                    path: (0, protopath_1.popSlash)(v.Prefix.slice(Prefix.length)),
                })),
                ...response.Contents.map((v) => ({
                    type: 'file',
                    path: v.Key.slice(Prefix.length),
                    size: v.Size,
                })),
            ];
            ContinuationToken = response.NextContinuationToken;
        } while (ContinuationToken);
        return list.sort((a, b) => a.path.localeCompare(b.path));
    }
    return {
        read,
        write,
        /*
        createReadStream,
        writeStream,
        */
        list,
    };
}
exports.create = create;
